/**
 * sjismemo.js
 *
 * @description    Shift-JIS pad - type memo and you can get SHIFT-JIS text file
 * @fileOverview   Shift-JIS pad
 * @author         ayati (Nobuaki Aono)
 * @version        0.1.3
 * @date           2015-03-31
 * @link           http://www.ayati.com/
 * @copyright      Copyright (c) 2015 ayati<info@ayati.com>
 * @license        licensed under the MIT license.
 *                  http://opensource.org/licenses/mit-license.php
 *
 * Require library:
 *   - Encoding.js
 *       https://github.com/polygonplanet/encoding.js
 */
function pad2(n) {  // always returns a string
      return (n < 10 ? '0' : '') + n;
}

function impfile(evt) {
  var file = evt.target.files;
  // FileReader
  var reader = new FileReader();
  reader.onload = function(ev){
      var buffer = reader.result;
      if (buffer) {
        // Shift_JIS Array
        var sjisArray = new Uint8Array(buffer);
        // Convert encoding to UNICODE (JavaScript Unicode Array).
        var unicodeArray = new ArrayBuffer();
        unicodeArray = Encoding.convert(sjisArray, {
            to: 'UNICODE',
            from: 'AUTO',
            type: 'arraybuffer'
        });
        // Join to string.
        var unicodeString = Encoding.codeToString(unicodeArray);
        // file to textarea
        document.getElementById('memo').value = unicodeString;
      }
    }
    reader.readAsArrayBuffer(evt.target.files[0]);
}

function saveFile(memoout) {
  // setting file name (memoYYYYMMDD.txt)
  var today=new Date();
  var filename = 'memo'+today.getFullYear()+pad2(today.getMonth() + 1)+pad2(today.getDate())+'.txt';

  // string to array 文字列を配列に変換
  var unicodeArray = Encoding.stringToCode(memoout);
//  var unicodeArray = promise1.then(Encoding.stringToCode(memoout));
  // encode shift-JIS encoding.jsでシフトJISに変換
  var sjisArray = Encoding.convert(unicodeArray, 'SJIS', 'UNICODE');

  // make file
  var len = sjisArray.length;
  var outBuffer = new ArrayBuffer(len);
  var outView = new Uint8Array(outBuffer);
  for(i=0 ; i<len ; ++i) {
      outView[i] = sjisArray[i];
  }

  // Create a FileWriter object
  if (memoout.length) {
    window.URL = window.URL || window.webkitURL;
    var blob = new Blob([outBuffer], {type: 'application/octet-stream'});
    var lnk = document.getElementById('download');
    lnk.download = filename;
    lnk.href = window.URL.createObjectURL(blob);
  }
  // if no memo
    else{
         document.getElementById('newline').textContent="no text";
  }
}

function saveutf8File(memoout) {
  // setting file name (memoYYYYMMDD.txt)
  var today=new Date();
  var filename = 'memo'+today.getFullYear()+pad2(today.getMonth() + 1)+pad2(today.getDate())+'.txt';

//  var outBuffer = unescape(encodeURIComponent(memoout));

  // Create a FileWriter object
  if (memoout.length) {
    window.URL = window.URL || window.webkitURL;
    var blob = new Blob([memoout], {type: 'text/plain'});
    var lnk = document.getElementById('download');
    lnk.download = filename;
    lnk.href = window.URL.createObjectURL(blob);
  }
  // if no memo
    else{
         document.getElementById('newline').textContent="no text";
  }
}

function autoSave() {
    var tempMemo = document.getElementById('memo').value;
    chrome.storage.local.set({tmp1:tempMemo}, function(){
        var localstorageStatus = document.getElementById('localstorageStatus');
        localstorageStatus.textContent = 'local auto saved.';
        setTimeout(function() {
          localstorageStatus.textContent = '';
        }, 750);
    });
}
function autoLoad() {
    chrome.storage.local.get("tmp1", function(r){
        document.getElementById('memo').value = r.tmp1;
    });
}
function clearMemo() {
    var tempMemo = document.getElementById('memo').value;
    chrome.storage.local.set({tmp1:tempMemo}, function(){
        document.getElementById('memo').value = "";
    });
}

function setMemostyle() {
      var memoarea = document.getElementById('memo');
      chrome.storage.sync.get({
          widthMemo:640,
          heightMemo:400,
          fontsizeMemo:10,
          fontfamilyMemo:"sans-serif",
          fontweightMemo:"normal"
        },function(r){
          memoarea.style.height = r.heightMemo+'px';
          memoarea.style.width = r.widthMemo+'px';
          memoarea.style.fontSize = r.fontsizeMemo+'pt';
          memoarea.style.fontFamily = r.fontfamilyMemo;
          memoarea.style.fontWeight = r.fontweightMemo;
      });
}

function openOptions() {
    autoSave;
    chrome.app.window.create('options.html', {
       'bounds': {'width': 680, 'height': 580}
    });
}

function save() {
  var memo = document.getElementById('memo').value;
  // LF to CRLF
//  memo = memo.replace(/\n/g, "\r\n");
  // CRLF to LF or CR (option)
  chrome.storage.sync.get({kaigyoMemo:"CRLF"}, function(item) {
    if (item.kaigyoMemo == "LF") {
         memo = memo.replace(/\r?\n/g,"\n");
         document.getElementById('newline').textContent="(LF)";
    }
    else if (item.kaigyoMemo == "CR") {
         memo = memo.replace(/\r?\n/g, "\r");
         document.getElementById('newline').textContent="(CR)";
    }
    else  {
         memo = memo.replace(/\n/g, "\r\n");
         document.getElementById('newline').textContent="";
    }
    chrome.storage.sync.get({encodeText:"Shift-JIS"}, function(item) {
        if (item.encodeText == "UTF-8") {
           saveutf8File(memo);
        }
        else {
            saveFile(memo);
        }
    });
  });
}

document.addEventListener('DOMContentLoaded', function() {
  // auto open
  chrome.storage.local.get({tmp1:""}, function(r){
      document.getElementById('memo').value = r.tmp1;
  });

  var saveNode = document.getElementById('save');
  saveNode.addEventListener('click', save, false);
  var reloadNode = document.getElementById('reload');
  reloadNode.addEventListener('click', autoLoad, false);
  var clearNode = document.getElementById('clear');
  clearNode.addEventListener('click', clearMemo, false);
  var optionsNode = document.getElementById('options');
  optionsNode.addEventListener('click', openOptions, false);
  document.getElementById('memo').focus();
  var infile = document.getElementById("selfile");
  infile.addEventListener('change', impfile, false);
  var memoarea = document.getElementById('memo');
  chrome.storage.sync.get({
      widthMemo:640,
      heightMemo:400,
      fontsizeMemo:10,
      fontfamilyMemo:"sans-serif",
      fontweightMemo:"normal"
    },function(r){
      memoarea.style.height = r.heightMemo+'px';
      memoarea.style.width = r.widthMemo+'px';
      memoarea.style.fontSize = r.fontsizeMemo+'pt';
      memoarea.style.fontFamily = r.fontfamilyMemo;
      memoarea.style.fontWeight = r.fontweightMemo;
  });
  // メッセージリスナー
  chrome.runtime.onMessage.addListener(function(request, sender, sendResponse) {
      if (request.name === "reloadSjispad") {
          var response = {data: "reloadOK"};
          sendResponse(response);
    }
  });
  // callback
  chrome.storage.onChanged.addListener(function(changes, namespace) {
    if (namespace == "sync") {
      if (changes.widthMemo) {
        document.getElementById('memo').style.width = changes.widthMemo.newValue+'px';
      }
      if (changes.heightMemo) {
        document.getElementById('memo').style.height = changes.heightMemo.newValue+'px';
      }
      if (changes.fontsizeMemo) {
        document.getElementById('memo').style.fontSize = changes.fontsizeMemo.newValue+'pt';
      }
      if (changes.fontfamilyMemo) {
        document.getElementById('memo').style.fontFamily = changes.fontfamilyMemo.newValue;
      }
      if (changes.fontweightMemo) {
        document.getElementById('memo').style.fontWeight = changes.fontweightMemo.newValue;
      }
    }
  });

}, false);

document.onkeyup = function(e) {
    var tempMemo = document.getElementById('memo').value;
    chrome.storage.local.set({tmp1:tempMemo}, function(){
        var localstorageStatus = document.getElementById('localstorageStatus');
        localstorageStatus.textContent = '.';
        setTimeout(function() {
          localstorageStatus.textContent = '';
        }, 100);
    });

}

document.onkeydown = function(e) {
// Ctrl+d    insert YYYY/MM/DD hh:mm:ss
// Ctrl+q    clr textarea
// Ctrl+i    open new text edit window
// Ctrl+s    make download link
    var shift, ctrl;
    var memotext = document.getElementById('memo').value;

       keycode = e.which;
        ctrl    = typeof e.modifiers == 'undefined' ? e.ctrlKey : e.modifiers & Event.CONTROL_MASK;
        shift   = typeof e.modifiers == 'undefined' ? e.shiftKey : e.modifiers & Event.SHIFT_MASK;

	    // キーコードの大文字を取得
	    keychar = String.fromCharCode(keycode).toUpperCase();

      if (ctrl) {
          if (keychar == "D") {
        // イベントの上位伝播を防止
    	        e.preventDefault();
              e.stopPropagation();
              var today=new Date();
              var instext =  today.getFullYear()+'/'+pad2(today.getMonth() + 1)+'/'+pad2(today.getDate())+' '+
                 pad2(today.getHours())+':'+pad2(today.getMinutes())+':'+pad2(today.getSeconds())+" \n";
              document.getElementById('memo').value = instext+memotext;
	        }
          else if (keychar == "Q") {
        // イベントの上位伝播を防止
    	        e.preventDefault();
              e.stopPropagation();
              document.getElementById('memo').value = '';
	        }
          else if (keychar == "I") {
        // イベントの上位伝播を防止
    	        e.preventDefault();
              e.stopPropagation();
              window.open('data:text/html ;charset=Shift-JIS , <html contenteditable>');
	        }
          else if (keychar == "S") {
        // イベントの上位伝播を防止
    	        e.preventDefault();
              e.stopPropagation();
              save();
	        }
	    }
}
